import 'package:http/http.dart';

/// An HTTP request to be made against the API.
abstract class HttpRequest {
  /// The route for this request.
  final String route;

  /// The method for this request.
  final String method;

  /// The query parameters for this request.
  final Map<String, String> queryParameters;

  /// The headers for this request.
  final Map<String, String> headers;

  /// Create a new [HttpRequest].
  ///
  /// {@macro http_request}
  HttpRequest(
    this.route, {
    this.method = 'GET',
    this.queryParameters = const {},
    this.headers = const {},
  });

  /// Transform this [HttpRequest] into a [BaseRequest] to be sent.
  BaseRequest prepare();

  Uri _getUri() => Uri.https(
        'commentanalyzer.googleapis.com',
        '/v1alpha1/$route',
        queryParameters.isNotEmpty ? queryParameters : null,
      );

  @override
  String toString() => 'HttpRequest($method $route)';
}

/// An [HttpRequest] with a JSON body.
class BasicRequest extends HttpRequest {
  /// The `Content-Type` header for JSON requests.
  static const jsonContentTypeHeader = {'Content-Type': 'application/json'};

  /// The JSON-encoded body of this request.
  ///
  /// Set to `null` to send no body.
  final String? body;

  /// Create a new [BasicRequest].
  BasicRequest(
    super.route, {
    this.body,
    super.method,
    super.queryParameters,
    super.headers,
  });

  @override
  Request prepare() {
    final request = Request(method, _getUri());
    request.headers.addAll(headers);

    if (body != null) {
      request.headers.addAll(jsonContentTypeHeader);
      request.body = body!;
    }

    return request;
  }
}
